# Lissajous figures for Pimoroni Pico Display 2 320x240 pixels
# Uses Pimoroni Pico Graphics system
# Tony Goodhew 21th February 2023
# Modified from https://www.instructables.com/Lissajous-Clock/
# https://academo.org/demos/lissajous-curves/
import machine
import time
import math
from pimoroni import Button
from picographics import PicoGraphics, DISPLAY_PICO_DISPLAY_2, PEN_P4
from pimoroni import RGBLED

# Set up the potentiometers in a list
pots = [] # Empty list of potentiometers
for i in range(3):
    pot = machine.ADC(i)
    pots.append(pot)

# Reduced colours to save RAM
display = PicoGraphics(display=DISPLAY_PICO_DISPLAY_2, pen_type=PEN_P4, rotate=0)
display.set_backlight(0.8)
display.set_font("bitmap8") # Lower case included

led = RGBLED(6, 7, 8)
led.set_rgb(0,0,0)     # Turn RGBLED OFF

display.set_font("bitmap8")

button_a = Button(12)
button_b = Button(13)
button_x = Button(14)
button_y = Button(15)

width  = 320  # Display size
height = 240
BLACK = display.create_pen(0,0,0)
w = display.create_pen(255,255,255)
r = display.create_pen(255,0,0)
g = display.create_pen(0,255,0)
b = display.create_pen(0,0,255)
y = display.create_pen(255,255,0)
c = display.create_pen(0,255,255)
m = display.create_pen(255,0,255)
gr = display.create_pen(180,180,180)
o = display.create_pen(255,128,30)
colours = [w,r,g,b,c,m,y,gr,o]
# ==== Board now setup ====

def lissajous(n,m,ph,w,amp,xoff,yoff):    
    # converts a value, n, into a lissajous pattern with complete lobes
    # n - nodes in x direction
    # m - nodes in y direction
    # f - phase shift - fi in degrees
    # w - width of curves
    # h - half amplitude
    # xoff - horizontal offset - centre of figure
    # yoff - vertical offset
    h = int(amp/2)
    w = int(w/2)
    t = int(n) % 2
    for a in range(722):
        j = a * int(m) + ph
        x = int(math.sin(j) * w) 
        g = a * int(n)

        if t != 0:
            y = int(math.cos(g) * h)
        else:
            y = int(math.sin(g) * h)
        display.pixel(x+xoff, y+yoff)
    return

def wait(d):
    display.update()
    time.sleep(d)
    display.set_pen(BLACK)
    display.clear()

def values(n,m,fi):
    display.set_pen(r)
    display.text("n = "+str(n), 250, 70, 200, 2)
    display.text("m = "+str(m), 250, 110, 200, 2)
    display.text("fi = "+str(fi), 250, 150, 200, 2)

# Function to get a number from the position of the potentiometer knob 
def get_pot_no(adc, maxx):
    reading = pots[adc].read_u16()
    # Adjust low end
    reading = reading - 700
    if reading < 0:
        reading = 0
    result = int((maxx * reading)/(65535-750)*1.01) # rescale
    # Adjust top end
    if result > maxx:
        result = maxx
    return result # Send answer back to main program

# Splash screen
display.set_pen(BLACK)
display.clear()
display.set_pen(y)
display.text("Lissajous Curves", 50, 70, 200, 4)
display.set_pen(w)
display.text("Tony Goodhew", 50, 160, 200, 3)
display.set_pen(b)
display.text("Potentiometer version", 50, 200, 200, 2)
display.update()
time.sleep(2)
display.set_pen(BLACK)
display.clear()
'''
f = 0
nn = [1,1,2,1,2,2,3,6,1,2,3,4,1,5]
mm = [1,3,7,9,3,5,2,5,2,5,5,7,6,1]
for p in range(len(nn)):
    n = nn[p]
    m = mm[p]
    display.set_pen(colours[p % 9])
    lissajous(n,m,0,200,200,101,119) # nodes,nodes,fi, width, amplitude, centre (x,y)
    values(n,m,f)
    wait(2)
 '''           

for count in range(200):    
    display.set_pen(g)    
    n = get_pot_no(0,15)+1
    m = get_pot_no(1,15)+1
    f1 = get_pot_no(2,360)
    f2 = f1 / 57.296
    display.set_pen(BLACK)
    display.clear()
    display.set_pen(g)
    lissajous(n,m,f2,200,200,101,119) # nodes, nodes,fi, width, amplitude, centre (x,y)      
    values(n,m,f1)
    display.set_pen(b)
    display.text("Twist the pots",245,200,80,2)
    display.update()

for count in range(10):
    step = 5
    for f in range(0,360,step):
        n = get_pot_no(0,15)+1
        m = get_pot_no(1,15)+1
        f2 = f / 57.296
        display.set_pen(BLACK)
        display.clear()
        display.set_pen(g)
        lissajous(n,m,f2,200,200,101,119) # nodes, nodes,fi, width, amplitude, centre (x,y)      
        values(n,m,f)
        display.set_pen(y)
        display.text("Auto Phase Shift",245,5,80,2)
        display.set_pen(b)
        display.text("Twist pots n and m",210,200,100,2)
        display.update()

display.set_pen(BLACK)
display.clear()
display.set_pen(y)
display.text("All Done", 50, 70, 200, 4)
display.update()
time.sleep(2)
display.set_pen(BLACK)
display.clear()
display.update()












































































































































































































































































































































